// agent.hxx: agent class.
// 
// Copyright (C) 2004 Frank Langbein <frank@langbein.org>
// All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
// 
// $Id: main.cc,v 2.7 2003/09/01 12:30:26 scmfcl Exp $

#ifndef AGENT_HXX
#define AGENT_HXX

#include "world.hxx"

#define AGENT_BUFFER 128

class World;

class Agent : public Object {
public:
  Agent (unsigned long aid, string const &n, World *w,
	 SimpleList<Agent> *agts);
  ~Agent ();
  void send (string const &str);
  void receive (string &str);
  string name () const;
  long positionX () const {
    return posx;
  }
  long positionY () const {
    return posy;
  }
  long orientation () const {
    return o;
  }
  void init ();
  void render () const;
  virtual GLfloat height () const {
    return 2.5 * sin (35 * M_PI / 180.0) * 2.0 + .3 * 2.0;
  }
  Agent *turn (World *w);
private:
  string nm, func;
  bool scheme;
  pid_t pid;
  bool broken;
  FILE *sndfp, *rcvfp;
  long posx, posy, o, id;
  GLfloat ambient[4], diffuse[4], specular[4];
  Object *carry;
  static char buffer[AGENT_BUFFER];
  static GLfloat bambient[4], bdiffuse[4], bspecular[4], bshine;
  static GLuint dl;
};

#endif // WORLD_HXX
    
/// Local Variables:          ***
/// compile-command: "make"   ***
/// End:                      ***
