/* config.h: Master configuration header file.
 * 
 * Copyright (C) 2004 Frank Langbein <frank@langbein.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: config.h,v 2.6 2003/09/01 12:30:25 scmfcl Exp $
 */

/* NOTE: This header file should be included first by all source files, but
 * not by any of the other header files. 
 *
 * There is no need to edit this file for changing the configuration. This
 * should always be done by the autoconf script configure and the header
 * file generated by that script.
 */

#ifndef CONFIG_H
#define CONFIG_H 1

#ifdef __cplusplus
using namespace std;
#endif /* __cplusplus */

/* Include the configuration parameters from configure/autoconf first. */
#ifdef HAVE_CONFIG_H
#include "c-auto.h"
#endif /* HAVE_CONFIG_H */

/* If debugging is not enabled by configure, define the no-debug symbols
 * for assert, ...etc. */
#ifndef DEBUG
#define NDEBUG
#define NO_DEBUG
#endif /* !DEBUG */

/* If we are using glibc this enables some useful extensions, otherwise
 * it should have no effect. */
#define _GNU_SOURCE 1

/* Standard C header files */

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif /* HAVE_STDIO_H */

#ifdef HAVE_MATH_H
#include <math.h>
#endif /* HAVE_MATH_H */

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif /* HAVE_FCNTL_H */

#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */

#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif /* HAVE_LIMITS_H */

#ifdef __cplusplus
/* We only need this in the C++ code and that way we avoid some warnings. */

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif /* HAVE_SYS_TIME_H */

#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif /* HAVE_SYS_STAT_H */

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif /* HAVE_SYS_TYPES_H */

#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif /* HAVE_SYS_WAIT_H */

#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif /* HAVE_SIGNAL_H */

#ifdef HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif /* HAVE_SYS_RESOURCE_H */

/* Standard C++ header files */

#ifdef HAVE_IOSTREAM
#include <iostream>
#endif /* HAVE_IOSTRAM */

#ifdef HAVE_FSTREAM
#include <fstream>
#endif /* HAVE_FSTREAM */

#endif /* __cplusplus */

/* Guile */
#ifdef HAVE_GUILE_GH_H
#include <guile/gh.h>
#endif /* HAVE_GUILE_GH_H */

/* Compiler characteristics. */

/* The NULL pointer should be defined on most systems. But there are
 * still some which don't provide it. */
#ifndef NULL
#ifdef __cplusplus
#define NULL 0
#else /* __cplusplus */
#define NULL (void *)0
#endif /* !__cplusplus */
#endif /* !NULL */

/* Define const to be empty if it is not already defined or provided
 * by the compiler. */
#if !defined(__STDC__) || !__STDC__
/* This is a separate conditional since some standard C systems reject
 * `defined (const)'. */
#ifndef const
#define const
#endif /* !const */
#endif /* !defined(__STDC__) || !__STDC__ */

/* Check if maximum for unsigned long is known. If not, use a reasonable
 * value. */
#ifndef ULONG_MAX
#ifdef __STDC__
#define ULONG_MAX 4294967295UL
#else /* __STDC__ */
#define ULONG_MAX   4294967295L
#endif /* __STDC__ */
#endif /* !ULONG_MAX */

/* Not-a-number definitions. */

#ifdef HAVE_NAN_H
/* The IEEE NAN code seems to be supported on this machine, so include
 * the nan.h header file. */
#include <nan.h>
#ifndef HAVE_ISNAN
/* If we don't have a isnan function, we simulate it with this comparison.
 * IEEE standard says that NAN != NAN. */
#define isnan(x) ((x) != (x))
#endif /* HAVE_ISNAN */
#else /* !HAVE_NAN_H */
#if !(defined(HAVE_ISNAN) && defined(NAN))
/* We don't have the nan.h header file and NAN and isnan is not
 * available. So we try to simulate the IEEE NAN code. But note that
 * this can have some strange effects even if the code is very close
 * to the real IEEE code. */
#ifdef WORDS_BIGENDIAN
#define sil_nan_bytes { (unsigned char) 0x7f, (unsigned char) 0xf8, \
                        (unsigned char) 0, (unsigned char) 0, \
                        (unsigned char) 0, (unsigned char) 0, \
                        (unsigned char) 0, (unsigned char) 0 }
#else /* !WORDS_BIGENDIAN */
#define sil_nan_bytes { (unsigned char) 0, (unsigned char) 0, \
                        (unsigned char) 0, (unsigned char) 0, \
                        (unsigned char) 0, (unsigned char) 0, \
                        (unsigned char) 0xf8, (unsigned char) 0x7f }
#endif /* !WORDS_BIGENDIAN */
static const unsigned char sil_nan[8] = sil_nan_bytes;
#ifdef NAN
#undef NAN
#endif /* NAN */
#define NAN (*(const double *) sil_nan)
#define USE_SIL_ISNAN 1
#endif /* !(defined(HAVE_ISNAN) && defined(NAN)) */
#endif /* !HAVE_NAN_H */

/* Assertions are too useful to not make them generally available. 
 * We only use them if debugging is enabled and if the system provides
 * them. We define an extra macro assert_exe(expr) which will always
 * execute expr, no matter if the assertions are enabled or not. But
 * the result will only be checked when the assertions are enabled. */
#ifdef HAVE_ASSERT_H

#include <assert.h> 

/* Execute assertion.
 *
 * This first executes the given expression even if assertions are
 * disabled. If the assertions are enabled, assert is called with the
 * result for the expression. This is used to run assertions on the
 * results of certain function calls without creating lots of extra
 * code storing these results. 
 */
#ifdef NO_DEBUG 
#define assert_exe(expr) expr
#else /* !NO_DEBUG */
#define assert_exe(expr) assert (expr)
#endif /* !NO_DEBUG */

#else /* !HAVE_ASSERT_H */

#define assert(expr)
#define assert_exe(expr) expr

#endif /* !HAVE_ASSERT_H */

/* OpenGL and GLUT header files. */

#ifdef __cplusplus
/* There is no real need to do this, but we only use OpenGL in the C++ code
 * and this avoids a few warnings. */

#ifdef HAVE_GL_GL_H
#include <GL/gl.h>
#endif /* HAVE_GL_GL_H */

#ifdef HAVE_GL_GLU_H
#include <GL/glu.h>
#endif /* HAVE_GL_GLU_H */

#ifdef HAVE_GL_GLX_H
#include <GL/glx.h>
#endif /* HAVE_GL_GLX_H */

#ifdef HAVE_GL_GLUT_H
#include <GL/glut.h>
#endif /* HAVE_GL_GLUT_H */

#endif /* __cplusplus */

/* Set RCS id.
 *
 * Set the RCS id for the module. This defines a variable
 * rcsid_##module which points to the RCS id string. This way of
 * setting the string will avoid any Warnings about it and still
 * provide all the RCS id strings in the binary.
 */
#define RCSID(module, idstr) const char *rcsid_##module = idstr

/* Precision */
#define DEFAULT_EPSILON 1e-10

#define RAND (static_cast<double> (random ()) / (static_cast<double> (RAND_MAX) + 1.0))

/* Mouse sensitivity */
#define DEFAULT_SENS_ANGLE 0.005
#define DEFAULT_SENS_POSITION 1.0

#endif /* !CONFIG_H */

/***                         ***
 *** Local Variables:        ***
 *** compile-command: "make" ***
 *** End:                    ***/
