// world.hxx: world class.
// 
// Copyright (C) 2004 Frank Langbein <frank@langbein.org>
// All rights reserved.
//
// This program is free software; you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
// 
// $Id: main.cc,v 2.7 2003/09/01 12:30:26 scmfcl Exp $

#ifndef WORLD_HXX
#define WORLD_HXX

typedef enum {
  CUBE = 0, CONE, SPHERE, TORUS, TEAPOT, DISC, CYL
} ObjectShape;

typedef enum {
  RED = 0, GREEN, BLUE, YELLOW, PURPLE, CYAN, BROWN, WHITE, BLACK
} ObjectColor;

class Object {
public:
  Object (string const &str);
  virtual ~Object () { }
  string const &name () const {
    return nm;
  }
  static void setup ();
  virtual void init () = 0;
  virtual void render () const = 0;
  virtual GLfloat height () const  = 0;
protected:
  static GLfloat ambient[4 * 9], diffuse[4 * 9], specular[4 * 9], shine;
  static string color[9], shape[7];
  static GLuint dl;
private:
  string nm;
};

class Box : public Object {
public:
  Box (ObjectShape s, ObjectColor c);
  ~Box () { }
  void init ();
  void render () const;
  GLfloat height () const {
    return 4.0;
  }
private:
  ObjectColor col;
  ObjectShape sh;
};

class Block : public Object {
public:
  Block (ObjectShape s, ObjectColor c);
  ~Block () { }
  void init ();
  void render () const;
  GLfloat height () const {
    return 4.0;
  }
private:
  ObjectColor col;
  ObjectShape sh;
};

class Trap : public Object {
public:
  Trap ();
  ~Trap () { }
  void init ();
  void render () const;
  GLfloat height () const {
    return 0.1;
  }
private:
  ObjectColor col;
  ObjectShape sh;
};

class Collect : public Object {
public:
  Collect (ObjectColor c);
  ~Collect () { }
  void init ();
  void render () const;
  GLfloat height () const {
    return 0.1;
  }
private:
  ObjectColor col;
  ObjectShape sh;
};

class Checkers : public Object {
public:
  Checkers (ObjectColor c);
  ~Checkers () { }
  void init ();
  void render () const;
  GLfloat height () const {
    return 0.1;
  }
private:
  ObjectColor col;
};

class World {
public:
  World (string world_type);
  ~World ();
  SimpleList<Object> *operator() (unsigned long x, unsigned long y) const {
    return grid[y * sX + x];
  }
  SimpleList<Object> *&operator() (unsigned long x, unsigned long y) {
    return grid[y * sX + x];
  }
  bool is_game () const {
    return g;
  }
  int check_win (unsigned long id);
  unsigned long sizeX () const {
    return sX;
  }
  unsigned long sizeY () const {
    return sY;
  }
  void init ();
  void render ();
private:
  void vacuum_world ();
  void scheckers_world ();
  SimpleList<Object> **grid;
  unsigned long sX, sY, count;
  GLfloat light_pos[4];
  GLuint dlW;
  bool g;
  static GLfloat light_diffuse[4], light_ambient[4], light_specular[4];
  static GLfloat light_attenuation;
  static GLfloat floor_diffuse1[4], floor_ambient1[4], floor_specular1[4];
  static GLfloat floor_shine1;
  static GLfloat floor_diffuse2[4], floor_ambient2[4], floor_specular2[4];
  static GLfloat floor_shine2;
};

#endif // WORLD_HXX
    
/// Local Variables:        ***
/// compile-command: "make" ***
/// End:                    ***
