# config/libs.m4                                        -*- Autoconf -*-
# Apsu autoconf library macro.
#
# This file is part of the apsu project.
# See http://www.langbein.org/software/apsu/.
#
# Copyright (C) 2004 Frank C. Langbein <frank@langbein.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
#  02111-1307, USA.
#
# $Id$

# AC_APSU_LIBRARY(LIBRARY-PACKAGE-NAME, 
#                 [HEADER, LIBRARY, FUNCTION, REQUIRED])
# ------------------------------------------------------
# Adds --with-$1, --with-$1-lib, --with-$1-inc user options to define
# library and header paths for library package $1. If any of these
# directories exist, then @$1_LIB@ contains the appropriate linker and
# @$1_INC@ contains the appropraite preprocessor flags. If optional
# arguments are given, then the macro tries to link a program with
# -lLIBRARY which calls the function FUNCTION and includes the header
# HEADER. If this fails, and required is set to yes, then configure
# fails. Note that without optional arguments @$1_LIB@ will not
# contain the -lLIBRARY flag! Also note that LIBRARY can be a space
# separated list of multiple libgraries, which will all be added
# appropriately to the linker flags. If everything was successful
# (with or without link test), a preprocessor symbol HAVE_$1_LIB
# will be defined.
AC_DEFUN([AC_APSU_LIBRARY],[dnl
  AC_REQUIRE([AC_PROG_CXX])
  AC_ARG_WITH([$1],
              AC_HELP_STRING([--with-$1=@<:@DIR@:>@],
                             [set base directory for $1 library (library in DIR/lib and headers in DIR/include)]),
              [AS_VAR_SET(ac_apsu_library_base_$1, $withval)],
              [AS_VAR_SET(ac_apsu_library_base_$1, none)])
  AC_ARG_WITH([$1-lib],
              AC_HELP_STRING([--with-$1-lib=@<:@DIR@:>@],
                             [set library directory for $1 library]),
              [AS_VAR_SET(ac_apsu_library_lib_$1, $withval)],
              [AS_VAR_SET(ac_apsu_library_lib_$1, none)])
  AC_ARG_WITH([$1-inc],
              AC_HELP_STRING([--with-$1-inc=@<:@DIR@:>@],
                             [set include directory for $1 library]),
              [AS_VAR_SET(ac_apsu_library_inc_$1, $withval)],
              [AS_VAR_SET(ac_apsu_library_inc_$1, none)])
  AC_MSG_CHECKING([for $1 library])
  AS_VAR_SET(ac_apsu_$1_ldflags, "")
  for l in "AS_VAR_GET(ac_apsu_library_lib_$1)" "AS_VAR_GET(ac_apsu_library_base_$1/lib)"
  do
    AS_IF([test "$l" != "none" -a "$l" != "none/lib" -a -d "$l"],
          [AS_VAR_SET(ac_apsu_$1_ldflags, 
                      ["AS_VAR_GET(ac_apsu_$1_ldflags) -L$l"])])
  done
  AC_MSG_RESULT(AS_VAR_GET(ac_apsu_$1_ldflags))
  AS_VAR_SET(AS_TR_CPP($1_LIB), AS_VAR_GET(ac_apsu_$1_ldflags))
  AC_SUBST(AS_TR_CPP($1_LIB))
  AC_MSG_CHECKING([for $1 headers])
  AS_VAR_SET(ac_apsu_$1_cppflags, "")
  for l in "AS_VAR_GET(ac_apsu_library_inc_$1)" "AS_VAR_GET($ac_apsu_library_base_$1)/include"
  do
    AS_IF([test "$l" != "none" -a "$l" != "none/lib" -a -d "$l"],
          [AS_VAR_SET(ac_apsu_$1_cppflags, 
                      ["AS_VAR_GET(ac_apsu_$1_cppflags) -I$l"])])
  done
  AC_MSG_RESULT(AS_VAR_GET(ac_apsu_$1_cppflags))
  AS_VAR_SET(AS_TR_CPP($1_INC), AS_VAR_GET(ac_apsu_$1_cppflags))
  AC_SUBST(AS_TR_CPP($1_INC))
  AS_VAR_SET(ac_aspu_def_$1, no)
  AS_IF([test "X$2" == X],
        [AS_IF([test "AS_VAR_GET(ac_apsu_library_inc_$1)" != none -o "AS_VAR_GET(ac_apsu_library_base)" != none -o "AS_VAR_GET(ac_apsu_library_lib_$1)" != none],
               [AS_VAR_SET(ac_apsu_def_$1, yes)])],
        [AS_VAR_SET(ac_apsu_$1_libs, "")
         for l in $3
         do
           AS_VAR_SET(ac_apsu_$1_libs, 
                      ["AS_VAR_GET(ac_apsu_$1_libs) -l$l"])
         done
         AS_VAR_SET(AS_TR_CPP($1_LIB), 
                    ["AS_VAR_GET(ac_apsu_$1_ldflags) AS_VAR_GET(ac_apsu_$1_libs)"])
         AS_VAR_SET(save_LDFLAGS, [AS_VAR_GET(LDFLAGS)])
         AS_VAR_SET(LDFLAGS, 
                    ["AS_VAR_GET(LDFLAGS) AS_VAR_GET(ac_apsu_$1_ldflags) AS_VAR_GET(ac_apsu_$1_libs)"])
         AS_VAR_SET(save_CPPFLAGS, [AS_VAR_GET(CPPFLAGS)])
         AS_VAR_SET(CPPFLAGS, 
                    ["AS_VAR_GET(CPPFLAGS) AS_VAR_GET(ac_apsu_$1_cppflags)"])
         AC_MSG_CHECKING([for usability of $1])
         AC_LINK_IFELSE(AC_LANG_PROGRAM([#include "$2"
                        void $4 (void);],[$4 ();]),
                        [AC_MSG_RESULT([yes])
                         AS_VAR_SET(ac_apsu_def_$1, yes),
                         AC_DEFINE(AS_TR_CPP(HAVE_$1_LIB), 1,
                                   [Define if $1 library is present])],
                        [AC_MSG_RESULT([no])
                         AS_IF([test "$5" = yes],
                               [AC_MSG_FAILURE([library $1 not usable])],
                               [AC_MSG_WARN([library $1 not usable])])])
         AS_VAR_SET(LDFLAGS, AS_VAR_GET(save_LDFLAGS))
         AS_VAR_SET(CPPFLAGS, AS_VAR_GET(save_CPPFLAGS))])
  AS_IF([test AS_VAR_GET(ac_apsu_def_$1) = yes],
        [AC_DEFINE(AS_TR_CPP(HAVE_$1_LIB), 1,
                   [Define if $1 library is present])])
])
