// pf/astar.hxx
// PF astar search.
//
// This file is part of the apsu project.
// See http://www.langbein.org/software/apsu/.
//
// Copyright (C) 2005 Frank C. Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// But WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef PF_ASTAR
#define PF_ASTAR 1

#include <math.h>

namespace PF {

  #include <stdlib.h>

  class Map;
  class Queue;

  class AStar {
  public:

    AStar (unsigned long sx, unsigned long sy,
	   unsigned long ex, unsigned long ey,
	   Map *map)
      : sx_ (sx),
	sy_ (sy),
	ex_ (ex),
	ey_ (ey),
	map_ (map),
	queue_ (NULL) {
    }

    ~AStar () {
    }

    bool search ();

    double heuristic (unsigned long x, unsigned long y) {
      double dx (static_cast<double> (x) - static_cast<double> (ex_));
      double dy (static_cast<double> (y) - static_cast<double> (ey_));
      return sqrt (dx * dx + dy * dy);
    }

  private:

    unsigned long sx_;
    unsigned long sy_;
    unsigned long ex_;
    unsigned long ey_;
    
    Map *map_;
    
    Queue *queue_;
  };

}

#endif // PF_ASTAR
