// pf/map.hxx
// PF map.
//
// This file is part of the apsu project.
// See http://www.langbein.org/software/apsu/.
//
// Copyright (C) 2005 Frank C. Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// But WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#ifndef PF_MAP
#define PF_MAP 1

namespace PF {

  typedef enum {
    UNDEFINED = 0,
    OCEAN     = 1,
    MOUNTAIN  = 10,
    FOREST    = 100,
    PLANE     = 1000
  } Terrain;

  class PF::Map {
  public:
    Map (unsigned long x,
	 unsigned long y,
	 Terrain b = OCEAN,
	 unsigned int ow = 1,
	 unsigned int mw = 1,
	 unsigned int fw = 1,
	 unsigned int pw = 1);
    ~Map ();
    Terrain &operator() (unsigned long const &x, unsigned long const &y) {
      assert (x < x_ && y < y_);
      return *(m_ + y * x_ + x);
    }
    void prev (unsigned long nx, unsigned long ny,
	       unsigned long &px, unsigned long &py) {
      px = *(px_ + ny * x_ + nx);
      py = *(py_ + ny * x_ + nx);
      return;
    }
    unsigned long size_x () const {
      return x_;
    }
    unsigned long size_y () const {
      return y_;
    }
    bool expand (unsigned long x, unsigned long y,
		 unsigned long &nx, unsigned long &ny,
		 double &c);
  private:

    bool get_pos (long* f, unsigned long x, unsigned long y,
		  unsigned long &nx, unsigned long &ny);

    unsigned long x_, y_;
    Terrain *m_;
    unsigned long *px_;
    unsigned long *py_;
    long *f_;
  };

}

#endif // PF_RENDER
