// src/map cxx
// PF map.
//
// This file is part of the apsu project.
// See http://www.langbein.org/software/apsu/.
//
// Copyright (C) 2005 Frank C. Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// But WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#include "config-pf.h"

ID(PF, pf_map, "$Id: main.c, v1.0 2003/09/01 scmfcl Exp $");

#include <time.h>
#include <stdlib.h>
#include <stdio.h>

#include "pf/map.hxx"

PF::Map::Map (unsigned long x,
	      unsigned long y,
	      Terrain b,
	      unsigned int ow,
	      unsigned int mw,
	      unsigned int fw,
	      unsigned int pw)
  : x_ (x),
    y_ (y) {

  unsigned long s = x * y;
  assert (s > 0UL);
  m_ = new Terrain[s];
  f_ = new long[s];
  px_ = new unsigned long[s];
  py_ = new unsigned long[s];
  
  for (unsigned long l = 0; l < s; ++l) {
    *(f_ + l) = 0L;
    *(px_ + l) = x_;
    *(py_ + l) = y_;
  }

  Terrain *p (m_);
  for (unsigned long l = 0; l < x_; ++l) {
    *(p++) = b;
  }
  p = m_;
  for (unsigned long l = 0; l < y_; ++l) {
    *p = b;
    p += x_;
  }

  time_t t (time (NULL));
  srandom (t);

  for (unsigned long l = 1; l < y_; ++l) {
    for (unsigned long k = 1; k < x_; ++k) {
      unsigned int neighbours ((*this) (k - 1, l - 1) + 
			       (*this) (k, l - 1) +
			       (*this) (k - 1, l) +
			       ((k < x_ - 1) ? ((*this) (k + 1, l - 1)) : 0));
      unsigned int no ( neighbours         % 10 + ow);
      unsigned int nm ((neighbours /   10) % 10 + mw);
      unsigned int nf ((neighbours /  100) % 10 + fw);
      unsigned int np ((neighbours / 1000) % 10 + pw);
      unsigned int total (no + nm + nf + np);
      double r (static_cast<double> (random ()) * static_cast<double> (total)
		/ static_cast<double> (RAND_MAX));
      if (r < no) {
	(*this) (k, l) = OCEAN;
      } else {
	r -= no;
	if (r < nm) {
	  (*this) (k, l) = MOUNTAIN;
	} else {
	  r -= nm;
	  (*this) (k, l) = (r < nf) ? FOREST : PLANE;
	}
      }
    }
  }

}

PF::Map::~Map () {
  delete[] m_;
  delete[] f_;
  delete[] px_;
  delete[] py_;
}

bool PF::Map::expand (unsigned long x, unsigned long y,
		      unsigned long &nx, unsigned long &ny,
		      double &c) {
  unsigned long p = y * x_ + x;
  long* flag (f_ + p);
  while (get_pos (flag, x, y, nx, ny)) {
    ++*flag;
    if (*(f_ + ny * x_ + nx) < 8L) {
      Terrain t ((*this) (nx, ny));
      if (t == PF::FOREST) {
	*(px_ + ny * x_ + nx) = x;
	*(py_ + ny * x_ + nx) = y;
	if (nx != x && ny != y) {
	  c = 2.0 * 1.4;
	} else {
	  c = 2.0;
	}
	return true;
      }
      if (t == PF::PLANE) {
	*(px_ + ny * x_ + nx) = x;
	*(py_ + ny * x_ + nx) = y;
	if (nx != x && ny != y) {
	  c = 1.4;
	} else {
	  c = 1.0;
	}
	c = 1.0;
	return true;
      }
    }
  }
  return false;
}

bool PF::Map::get_pos (long* f, unsigned long x, unsigned long y,
		       unsigned long &nx, unsigned long &ny) 
{
  while (*f < 8L) {
    switch (*f) {
    case 0:
      if (x > 0 && y > 0) {
	nx = x - 1;
	ny = y - 1;
	return true;
      }
      break;
    case 1:
      if (y > 0) {
	nx = x;
	ny = y - 1;
	return true;
      }
      break;
    case 2:
      if (x < x_ - 1 && y > 0) {
	nx = x + 1;
	ny = y - 1;
	return true;
      }
      break;
    case 3:
      if (x < x_ - 1) {
	nx = x + 1;
	ny = y;
	return true;
      }
      break;
    case 4:
      if (x < x_ - 1 && y < y_ - 1) {
	nx = x + 1;
	ny = y + 1;
	return true;
      }
      break;
    case 5:
      if (y < y_ - 1) {
	nx = x;
	ny = y + 1;
	return true;
      }
      break;
    case 6:
      if (x > 0 && y < y_ - 1) {
	nx = x - 1;
	ny = y + 1;
	return true;
      }
      break;
    case 7:
      if (x > 0) {
	nx = x - 1;
	ny = y;
	return true;
      }
      break;
    }
    ++(*f);
  }
  return false;
}
