// src/queue.cxx
// PF queue.
//
// This file is part of the apsu project.
// See http://www.langbein.org/software/apsu/.
//
// Copyright (C) 2005 Frank C. Langbein <frank@langbein.org>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// But WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
// 02111-1307, USA.

#include "config-pf.h"

ID(PF, pf_queue, "$Id: main.c, v1.0 2003/09/01 scmfcl Exp $");

#include <stdio.h>

#include "pf/queue.hxx"

PF::Queue::~Queue () {
  while (head_) {
    Node *n (head_->next_);
    delete head_;
    head_ = n;
  }
}

void PF::Queue::push (unsigned long x, unsigned long y, double g, double f) {
  if (!head_) {
    head_ = new Node (x, y, g, f, NULL);
    return;
  }
  if (f < head_->f_) {
    head_ = new Node (x, y, g, f, head_);
    return;
  }
  Node *n (head_);
  while (n->next_) {
    if (f < n->next_->f_) {
      n->next_ = new Node (x, y, g, f, n->next_);
      return;
    }
    n = n->next_;
  }
  n->next_ = new Node (x, y, g, f, n->next_);
  return;
}


bool PF::Queue::pop (unsigned long &x, unsigned long &y, double &g,
		     double &f) {
  if (!head_) {
    return false;
  }

  x = head_->x_;
  y = head_->y_;
  g = head_->g_;
  f = head_->f_;
  Node *n_ (head_);
  head_ = head_->next_;
  delete n_;
  return true;
}
